#include <gnome.h>
#include <applet-widget.h>

#include "euterpe.h"
#include "media.h"

char *archivePath;
char *actionInit, *actionFirst, *actionRest, *actionFini;

static GtkWidget *applet;
static GtkWidget *locText, *initText, *firstText, *restText, *finiText;

void euterpe_prop_apply(GnomePropertyBox *propBox, 
												gint arg1, 
												gpointer user_data) {
  gchar *newVal, *oldVal;

	if (arg1 != -1) {
		return;
	}

	/* We do this backwards because media_load_archives uses archivePath */
	oldVal = archivePath;
	archivePath = gtk_editable_get_chars(GTK_EDITABLE(locText), 0, -1);
	if (strcmp(archivePath, oldVal) != 0) {
		media_load_archives();
	}
	if (oldVal != NULL) {
		g_free(oldVal);
	}

	newVal = gtk_editable_get_chars(GTK_EDITABLE(initText), 0, -1);
	if (actionInit != NULL) {
		g_free(actionInit);
	}
	actionInit = newVal;

	newVal = gtk_editable_get_chars(GTK_EDITABLE(firstText), 0, -1);
	if (actionFirst != NULL) {
		g_free(actionFirst);
	}
	actionFirst = newVal;

	newVal = gtk_editable_get_chars(GTK_EDITABLE(restText), 0, -1);
	if (actionRest != NULL) {
		g_free(actionRest);
	}
	actionRest = newVal;

	newVal = gtk_editable_get_chars(GTK_EDITABLE(finiText), 0, -1);
	if (actionFini != NULL) {
		g_free(actionFini);
	}
	actionFini = newVal;

	applet_widget_sync_config(APPLET_WIDGET(applet));
}

void euterpe_menu_prop(AppletWidget *applet, gpointer data) {
  GtkWidget *propertiesDialog;
  GtkWidget *notebook;
  GtkWidget *vbox1;
  GtkWidget *locHbox, *initHbox, *firstHbox, *restHbox, *finiHbox;
  GtkWidget *locLabel, *initLabel, *firstLabel, *restLabel, *finiLabel;
	GtkWidget *tabLabel;

  propertiesDialog = gnome_property_box_new();
  gtk_object_set_data(GTK_OBJECT(propertiesDialog), 
											"propertiesDialog", 
											propertiesDialog);

  notebook = GNOME_PROPERTY_BOX(propertiesDialog)->notebook;
  gtk_object_set_data(GTK_OBJECT(propertiesDialog), "notebook", notebook);
  gtk_widget_show(notebook);

  vbox1 = gtk_vbox_new(FALSE, 10);
  gtk_widget_ref(vbox1);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "vbox1", 
													 vbox1,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(vbox1);
  gtk_container_add(GTK_CONTAINER(notebook), vbox1);

	/**** Location ****/
  locHbox = gtk_hbox_new(FALSE, 10);
  gtk_widget_ref(locHbox);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "locHbox", 
													 locHbox,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(locHbox);
  gtk_box_pack_start(GTK_BOX(vbox1), locHbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(locHbox), 10);

  locLabel = gtk_label_new("Media archive path");
  gtk_widget_ref(locLabel);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "locLabel", 
													 locLabel,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(locLabel);
  gtk_box_pack_start(GTK_BOX(locHbox), locLabel, FALSE, FALSE, 1);

  locText = gtk_entry_new();
  gtk_widget_ref(locText);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "locText", 
													 locText,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_signal_connect_object(GTK_OBJECT(locText), 
														"changed", 
														GTK_SIGNAL_FUNC(gnome_property_box_changed), 
														GTK_OBJECT(propertiesDialog));
  gtk_widget_show(locText);
  gtk_box_pack_start(GTK_BOX(locHbox), locText, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(locText), archivePath);

	/**** Initial action ****/
  initHbox = gtk_hbox_new(FALSE, 10);
  gtk_widget_ref(initHbox);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "initHbox", 
													 initHbox,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(initHbox);
  gtk_box_pack_start(GTK_BOX(vbox1), initHbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(initHbox), 10);

  initLabel = gtk_label_new("Initial action");
  gtk_widget_ref(initLabel);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "initLabel", 
													 initLabel,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(initLabel);
  gtk_box_pack_start(GTK_BOX(initHbox), initLabel, FALSE, FALSE, 1);

  initText = gtk_entry_new();
  gtk_widget_ref(initText);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "initText", 
													 initText,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_signal_connect_object(GTK_OBJECT(initText), 
														"changed", 
														GTK_SIGNAL_FUNC(gnome_property_box_changed), 
														GTK_OBJECT(propertiesDialog));
  gtk_widget_show(initText);
  gtk_box_pack_start(GTK_BOX(initHbox), initText, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(initText), actionInit);

	/**** First result action ****/
  firstHbox = gtk_hbox_new(FALSE, 10);
  gtk_widget_ref(firstHbox);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "firstHbox", 
													 firstHbox,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(firstHbox);
  gtk_box_pack_start(GTK_BOX(vbox1), firstHbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(firstHbox), 10);

  firstLabel = gtk_label_new("First result action");
  gtk_widget_ref(firstLabel);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "firstLabel", 
													 firstLabel,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(firstLabel);
  gtk_box_pack_start(GTK_BOX(firstHbox), firstLabel, FALSE, FALSE, 1);

  firstText = gtk_entry_new();
  gtk_widget_ref(firstText);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "firstText", 
													 firstText,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_signal_connect_object(GTK_OBJECT(firstText), 
														"changed", 
														GTK_SIGNAL_FUNC(gnome_property_box_changed), 
														GTK_OBJECT(propertiesDialog));
  gtk_widget_show(firstText);
  gtk_box_pack_start(GTK_BOX(firstHbox), firstText, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(firstText), actionFirst);

	/**** Other results action ****/
  restHbox = gtk_hbox_new(FALSE, 10);
  gtk_widget_ref(restHbox);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "restHbox", 
													 restHbox,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(restHbox);
  gtk_box_pack_start(GTK_BOX(vbox1), restHbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(restHbox), 10);

  restLabel = gtk_label_new("Other results action");
  gtk_widget_ref(restLabel);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "restLabel", 
													 restLabel,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(restLabel);
  gtk_box_pack_start(GTK_BOX(restHbox), restLabel, FALSE, FALSE, 1);

  restText = gtk_entry_new();
  gtk_widget_ref(restText);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "restText", 
													 restText,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_signal_connect_object(GTK_OBJECT(restText), 
														"changed", 
														GTK_SIGNAL_FUNC(gnome_property_box_changed), 
														GTK_OBJECT(propertiesDialog));
  gtk_widget_show(restText);
  gtk_box_pack_start(GTK_BOX(restHbox), restText, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(restText), actionRest);

	/**** Final action ****/
  finiHbox = gtk_hbox_new(FALSE, 10);
  gtk_widget_ref(finiHbox);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "finiHbox", 
													 finiHbox,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(finiHbox);
  gtk_box_pack_start(GTK_BOX(vbox1), finiHbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(finiHbox), 10);

  finiLabel = gtk_label_new("Final action");
  gtk_widget_ref(finiLabel);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "finiLabel", 
													 finiLabel,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(finiLabel);
  gtk_box_pack_start(GTK_BOX(finiHbox), finiLabel, FALSE, FALSE, 1);

  finiText = gtk_entry_new();
  gtk_widget_ref(finiText);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "finiText", 
													 finiText,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_signal_connect_object(GTK_OBJECT(finiText), 
														"changed", 
														GTK_SIGNAL_FUNC(gnome_property_box_changed), 
														GTK_OBJECT(propertiesDialog));
  gtk_widget_show(finiText);
  gtk_box_pack_start(GTK_BOX(finiHbox), finiText, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(finiText), actionFini);

	/**** Tab label ****/
  tabLabel = gtk_label_new("General");
  gtk_widget_ref(tabLabel);
  gtk_object_set_data_full(GTK_OBJECT(propertiesDialog), 
													 "tabLabel", 
													 tabLabel,
                           (GtkDestroyNotify)gtk_widget_unref);
  gtk_widget_show(tabLabel);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook), 
														 gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
																											 0), 
														 tabLabel);

  gtk_signal_connect(GTK_OBJECT(propertiesDialog), 
										 "apply",
										 GTK_SIGNAL_FUNC(euterpe_prop_apply),
										 NULL);

  gtk_widget_show(propertiesDialog);
}

void euterpe_menu_about(AppletWidget *applet, gpointer data) {
  GtkWidget *about;
  static const gchar *authors[1];
  authors[0] = "Matthew Dockrey <euterpe@cyphertext.net>";
  authors[1] = NULL;

  about = gnome_about_new("Euterpe Media Searcher", VERSION,
													"(C) 2001",
													authors,
													"Euterpe is a media archive searching applet. It takes a search string from either the current selection or a query dialog and searches for matching file names, playing any it finds.",
													NULL);

  gtk_widget_show(about);
}

void euterpe_menu_reload(AppletWidget *applet, gpointer data) {
	media_load_archives();
}

void euterpe_menu_query(AppletWidget *applet, gpointer data) {
	GtkWidget *dialog, *label, *queryText;
	int button;
	gchar *query;

	dialog = gnome_dialog_new("Query",
														GNOME_STOCK_BUTTON_OK,
														GNOME_STOCK_BUTTON_CANCEL,
														NULL);

	label = gtk_label_new("Search string:");
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), 
										 label, 
										 TRUE, 
										 TRUE, 
										 0);
  gtk_widget_show(label);

	queryText = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), 
										 queryText, 
										 TRUE, 
										 TRUE, 
										 0);
  gtk_widget_show(queryText);

	gnome_dialog_close_hides(GNOME_DIALOG(dialog), TRUE);
	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
	button = gnome_dialog_run_and_close(GNOME_DIALOG(dialog));

	switch (button) {
	case 0:
		query = gtk_editable_get_chars(GTK_EDITABLE(queryText), 0, -1);
		media_search(query);
		g_free(query);
		break;
	default:
		break;
	}

	gtk_widget_destroy(dialog);
}

void euterpe_get_selection(GtkWidget *widget) {
  static GdkAtom targets_atom = GDK_NONE;

  /* Get the atom corresponding to the string "TARGETS" */
  if (targets_atom == GDK_NONE) {
    targets_atom = gdk_atom_intern("STRING", FALSE);
	}

  /* And request the "TARGETS" target for the primary selection */
  gtk_selection_convert(widget, GDK_SELECTION_PRIMARY, targets_atom,
												GDK_CURRENT_TIME);
}

void euterpe_selection_received(GtkWidget *widget, GtkSelectionData *selection_data, 
                         gpointer data) {

  /* **** IMPORTANT **** Check to see if retrieval succeeded  */
  if (selection_data->length < 0) {
      g_print("Selection retrieval failed\n");
      return;
	}

  /* Make sure we got the data in the expected form */
  if (selection_data->type != GDK_SELECTION_TYPE_STRING) {
      g_print("Selection \"TARGETS\" was not returned as a string!\n");
      return;
	}

	media_search(selection_data->data);
}

static gint euterpe_save_session(GtkWidget *w,
																 const char *privcfgpath,
																 const char *globcfgpath) {

	gnome_config_set_string(PACKAGE "/Properties/archivepath", archivePath);
	gnome_config_set_string(PACKAGE "/Properties/actioninit", actionInit);
	gnome_config_set_string(PACKAGE "/Properties/actionfirst", actionFirst);
	gnome_config_set_string(PACKAGE "/Properties/actionrest", actionRest);
	gnome_config_set_string(PACKAGE "/Properties/actionfini", actionFini);

	gnome_config_sync();
	gnome_config_drop_all();

	return FALSE;
}

int main(int argc, char **argv) {
	GtkWidget *button;
	GtkWidget *buttonBox;
	GtkWidget *pixmapwid;
	GdkPixmap *pixmap;
	GdkBitmap *bitmap;
	GdkImlibImage *img;

	applet_widget_init(PACKAGE, 
										 VERSION, 
										 argc, 
										 argv,
										 NULL, 
										 0, 
										 NULL);

	applet = applet_widget_new(PACKAGE);
	if (!applet) {
		g_error("Can't create applet!\n");
	}

	archivePath = gnome_config_get_string(PACKAGE "/Properties/archivepath=/mnt/public/Media/MP3:/tmp/download");
	actionInit = gnome_config_get_string(PACKAGE "/Properties/actioninit=/usr/bin/xmms --stop");
	actionFirst = gnome_config_get_string(PACKAGE "/Properties/actionfirst=/usr/bin/xmms --play");
	actionRest = gnome_config_get_string(PACKAGE "/Properties/actionrest=/usr/bin/xmms --enqueue");
	actionFini = gnome_config_get_string(PACKAGE "/Properties/actionfini=");

	media_load_archives();

	button = gtk_button_new();
	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);

	buttonBox = gtk_hbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(buttonBox), 0);

	img = gdk_imlib_load_image(IMAGE_PATH);
	gdk_imlib_render(img, img->rgb_width, img->rgb_height);
	pixmap = gdk_imlib_move_image(img);
	bitmap = gdk_imlib_move_mask(img);
	pixmapwid = gtk_pixmap_new(pixmap, bitmap);

	gtk_box_pack_start(GTK_BOX(buttonBox), pixmapwid, TRUE, TRUE, 0);
	gtk_widget_show(pixmapwid); 

	gtk_container_add(GTK_CONTAINER(button), buttonBox);
	gtk_widget_show(buttonBox);
	applet_widget_add(APPLET_WIDGET(applet), button);
	gtk_widget_show(button);

 	gtk_signal_connect(GTK_OBJECT(button), 
										 "clicked",
										 GTK_SIGNAL_FUNC(euterpe_get_selection), 
										 NULL);	
	gtk_signal_connect(GTK_OBJECT(button), 
										 "selection_received",
										 GTK_SIGNAL_FUNC(euterpe_selection_received), 
										 NULL);
	gtk_signal_connect(GTK_OBJECT(applet),
										 "save_session",
										 GTK_SIGNAL_FUNC(euterpe_save_session),
										 NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
																				"about",
																				GNOME_STOCK_MENU_ABOUT,
																				"About",
																				&euterpe_menu_about,
																				NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
																				"properties",
																				GNOME_STOCK_MENU_PROP,
																				"Properties",
																				&euterpe_menu_prop,
																				NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
																				"reload",
																				GNOME_STOCK_MENU_REFRESH,
																				"Reload Cache",
																				&euterpe_menu_reload,
																				NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
																				"query",
																				GNOME_STOCK_MENU_SEARCH,
																				"Query Dialog",
																				&euterpe_menu_query,
																				NULL);

	applet_widget_set_tooltip(APPLET_WIDGET(applet), "Euterpe Media Searcher");
	gtk_widget_show(applet);

	applet_widget_gtk_main();

	return 0;
}
